// 'currencies-controller.component.js' file
import React, {Component} from "react";
import {withRouter} from "react-router-dom";

import Swal from "sweetalert2";

import './currencies-controller.component.css'; // webpack

class CurrenciesController extends Component {
    _initialization() {
        return {
            new_currency: {
                common_symbol: "e.g., $",
                common_name: "e.g., Dollar",
                description: "...",
                iso_code: "Three digits excluding '000'",
                iso_code_validity: false,
                iso_symbol: "Three upper case letters",
                iso_symbol_validity: false,
                substitution_date: null
            }
        };
    }

    constructor(props) {
        super(props);
        /* HTML5 Constraint Validation API */
        // const iso_code = window.document.getElementById("iso_code"); // No! Instead:
        this.iso_code = React.createRef(); // Direct access of DOM element (using 'current') fails because the DOM is not yet loaded...
        this.my_form = React.createRef(); // Direct access of DOM element (using 'current') fails because the DOM is not yet loaded...
        this.my_form_validity = false;
        this.state = this._initialization(); // "controlled components" mode...
    }

    check_iso_symbol = function (event) { /* ES7 instance attribute outside constructor -> supported by Babel */
        const iso_symbol = event.target.value;
        this.setState(prior_state => { // <- 'event.target.value' is unknown because updater function is asynchronous!
            const new_currency = Object.assign({}, prior_state.new_currency);
            new_currency.iso_symbol = iso_symbol.toUpperCase();
            new_currency.iso_symbol_validity = CurrenciesController.ISO_symbol_pattern_as_RegExp.test(new_currency.iso_symbol);
            return {new_currency};
        });
    }

    componentDidMount() { // Hook: close to 'ngOnInit' in Angular...
        /* HTML5 Constraint Validation API */
        this.iso_code.current.addEventListener('input', (event) => { // Each new typed character...
            this.setState(prior_state => ({
                new_currency: {
                    ...prior_state.new_currency, // Copy all other key-value pairs of 'new_currency' object...
                    iso_code_validity: true // A priori, no problem...
                }
            }));
            this.iso_code.current.setCustomValidity(""); // A priori, no problem...
            this.iso_code.current.checkValidity(); // Send 'invalid' if 'false'...
        });
        this.iso_code.current.addEventListener('invalid', (event) => { // Called at form submission time as well...
            // window.alert("'invalid': " + event.target.value);
            this.setState(prior_state => ({
                new_currency: {
                    ...prior_state.new_currency, // Copy all other key-value pairs of 'new_currency' object...
                    iso_code_validity: false // Problem...
                }
            }));
            if (this.iso_code.current.value === "")
                this.iso_code.current.setCustomValidity("Empty ISO code is not permitted...");
            else this.iso_code.current.setCustomValidity("3 digits excluding '000'");
        });
    }

    componentDidUpdate(prior_props, prior_state, getSnapshotBeforeUpdate) { // Hook...
        window.console.assert(getSnapshotBeforeUpdate === "Message to be checked in 'componentDidUpdate'");
        // this.my_form.current.reportValidity(); // "Error: Maximum update depth exceeded"
        if (this.my_form_validity)
            Swal.fire({
                position: 'top-end',
                icon: 'success',
                title: '(form valid) -> \'componentDidUpdate\'...',
                showConfirmButton: false,
                timer: 1500
            });
    }

    getSnapshotBeforeUpdate(prior_props, prior_state) { // Before the DOM is updated...
        // this.my_form.current.reportValidity(); // "Error: Maximum update depth exceeded"
        if (this.my_form_validity)
            window.console.log("'getSnapshotBeforeUpdate': " + this.my_form_validity);
        return "Message to be checked in 'componentDidUpdate'";
    }

    record_currency(event) {
        window.console.assert(event.target instanceof HTMLFormElement);
        event.preventDefault(); // This prevents the page from being refreshed...
        /* Add new data in 'Currencies.Currencies'... */
        /* Add new corresponding route in 'Currencies_menu.Navigations'... */
        const next_round = Math.floor(Math.random() * 2); // For the fun: '0' or '1'...
        if (next_round === 0) {
            Swal.fire({
                position: 'top-end',
                icon: 'success',
                title: '(form reset) -> "New" is following navigation...',
                showConfirmButton: false,
                timer: 1500
            });
            // Next round:
            this.setState(this._initialization()); // controlled-component way...
            event.target.reset(); // uncontrolled-component way...
        } else {
            window.console.assert(next_round === 1);
            Swal.fire({
                position: 'top-end',
                icon: 'success',
                title: '(form quit) -> "Currencies" is following navigation...',
                showConfirmButton: false,
                timer: 1500
            });
            // Next round:
            this.props.history.push('/Currencies'); // 'export default withRouter(CurrenciesController);'
        }
    }

    reset_common_name = function () { /* ES7 instance attribute outside constructor -> supported by Babel */
        // https://stackoverflow.com/questions/43638938/updating-an-object-with-setstate-in-react
        this.setState(prior_state => { // https://reactjs.org/docs/react-component.html#setstate
            const new_currency = Object.assign({}, prior_state.new_currency);
            new_currency.common_name = "";
            return {new_currency};
        });
    }
    set_common_name = function (event) { /* ES7 instance attribute outside constructor -> supported by Babel */
        const common_name = event.target.value;
        this.setState(prior_state => ({ // <- 'event.target.value' is unknown because updater function is asynchronous!
            new_currency: {
                ...prior_state.new_currency, // Copy all 'new_currency' key-value pairs...
                common_name: common_name.toUpperCase()
            }
        }));
    }

    render() { // This seems the best pattern (i.e., called inside 'render'):
        this.my_form_validity = this.state.new_currency.common_name.length > 0 && /* 'required' */
            this.state.new_currency.common_symbol.length > 0 && /* 'required' */
            this.state.new_currency.iso_code_validity &&
            this.state.new_currency.iso_symbol_validity;
        window.console.assert(!this.my_form_validity || this.my_form.current.reportValidity()); /* 'this.my_form_validity' => 'this.my_form.current.reportValidity()' */
        return (
            /*JSX fragment*/ <>
                <form action="" id="my_form" name="my_form"
                      onSubmit={this.record_currency.bind(this)}
                      ref={this.my_form}> {/* 'novalidate' -> HTML5 Constraint Validation API is inhibited... ('noValidate' in JSX?) */}
                    <p>
                        <label>Common name:
                            <input onChange={this.set_common_name.bind(this)}
                                   placeholder={this.state.new_currency.common_name} required type="text"
                                   value={this.state.new_currency.common_name}/>
                            <button onClick={this.reset_common_name.bind(this)} type="button">Reset</button>
                        </label>
                    </p>
                    <p>
                        <label>Common symbol:
                            <input placeholder={this.state.new_currency.common_symbol} required type="text"/>
                        </label>
                    </p>
                    <p>
                        <label>Description:
                            <input placeholder={this.state.new_currency.description} type="text"/>
                        </label>
                    </p>
                    <p>
                        <label>ISO code: {/* HTML5 Constraint Validation API through 'ref={this.iso_code}' */}
                            <input pattern={CurrenciesController.ISO_code_pattern_as_String_}
                                   placeholder={this.state.new_currency.iso_code}
                                   ref={this.iso_code} required type="text"/>
                        </label>
                    </p>
                    <p>
                        <label>ISO symbol: {/* Custom validation... */}
                            <input onChange={this.check_iso_symbol.bind(this)}
                                   placeholder={this.state.new_currency.iso_symbol}
                                   required
                                   type="text"
                                   value={this.state.new_currency.iso_symbol}/>
                        </label>
                        {/*Conditional rendering*/}
                        {!this.state.new_currency.iso_symbol_validity &&
                        <small>
                            At least 3, at most 3 capital letters...
                        </small>}
                    </p>
                    <p>
                        <label>Substitution date:
                            <input list="my_dates" name="substitution_date" type="date"/>
                        </label>
                    </p>
                </form>
                <datalist id="my_dates"> { /*Problem with Firefox while Chrome is OK...*/}
                    <option value="2020-01-11"/>
                    <option value="2020-02-22"/>
                    <option value="2020-04-15"/>
                    <option value="2020-07-28"/>
                    <option value="2020-12-06"/>
                </datalist>
                <p>
                    <button disabled={!this.my_form_validity} form="my_form" type="submit">Record currency...</button>
                </p>
            </> /*JSX fragment*/
        );
    }
}

// 'pattern' property of 'input' requires string:
CurrenciesController.ISO_code_pattern_as_String = "^\\d{3}$"; // 3 digits, exactly...
CurrenciesController.ISO_code_pattern_as_String_ = "^(?!000)(\\d{3}$)"; // 3 digits, exactly excluding '000'...
// CurrenciesController.ISO_code_pattern_as_RegExp = /^\d{3}$/; // 3 digits, exactly...
CurrenciesController.ISO_symbol_pattern_as_RegExp = new RegExp("^[A-Z]{3}$"); // 3 upper case letters, exactly...

export default withRouter(CurrenciesController);
