import React, {Component} from "react";
import {
    BrowserRouter as Router,
    Link,
    Redirect,
    Route,
    // Switch
} from "react-router-dom";

import './currencies-menu.component.css'; // webpack!

import Currencies from './currencies.component';
import CurrenciesController from './currencies-controller.component';
import CurrenciesInformation from './currencies-information.component';
import CurrenciesInformation_ from './currencies-information.component_';
import CurrenciesWrapper from './currencies-wrapper.component';

export default class CurrenciesMenu extends Component {
    static _Links = [
        {// eslint-disable-next-line
            path: '/' + 'Currencies',
            component: CurrenciesWrapper,
            data: {material_icon: 'payment'}
        },
        {// eslint-disable-next-line
            path: '/' + 'New',
            component: CurrenciesController,
            data: {material_icon: 'fiber_new'}
        },
        {// eslint-disable-next-line
            path: '/' + 'Currencies' + '/' + `${Currencies.Currencies[Currencies.Dollar].iso_code}`,
            component: CurrenciesInformation,
            data: {iso_code: Currencies.Currencies[Currencies.Dollar].iso_code, material_icon: 'attach_money'}
        },
        {// eslint-disable-next-line
            path: '/' + 'Currencies' + '/' + `${Currencies.Currencies[Currencies.Euro].iso_code}`,
            component: CurrenciesInformation, // => 4th component!
            data: {iso_code: Currencies.Currencies[Currencies.Euro].iso_code, material_icon: 'euro_symbol'}
        }
    ];
    static _Components = [...new Set(CurrenciesMenu._Links.map(link => link.component))]; // 3 or 4...
    static _Routes = CurrenciesMenu._Links.filter(link => {
        const index = CurrenciesMenu._Components.findIndex(component => component === link.component);
        CurrenciesMenu._Components.splice(index, 1);
        return index !== -1; // Found...
    });

    // https://fr.reactjs.org/docs/context.html
    // Single argument is default value:
    static Navigation_context = React.createContext(CurrenciesMenu._Links[Currencies.Dollar].path);

    render() {
        return (
            <Router>
                <nav className="currencies-menu">
                    { // <- JSX evaluate exp.
                        // 4 links:
                        CurrenciesMenu._Links.map(link => ( // '<nav *ngFor="let navigation of navigations">'
                            <Link className="mat-raised-button" to={link.path}>
                                {/* Ligature principle (https://alistapart.com/article/the-era-of-symbol-fonts/): */}
                                <i className="currencies material-icons">{link.data.material_icon}</i>
                            </Link>
                        ))
                    }
                </nav>
                { // <- JSX evaluate exp.
                    // 3 routes:
                    CurrenciesMenu._Routes.map(route => (
                        <Route component={route.component}
                               exact
                               path={("iso_code" in route.data) ? CurrenciesMenu._Routes[0].path + "/:iso_code" : route.path}>
                        </Route>
                    ))
                }
                <Route exact path="/">{true ? <Redirect to={CurrenciesMenu._Routes[0].path}/> :
                    <h1>No 'Redirect' yet...</h1>}</Route>
            </Router>
        );
    }
}
