// currencies-wrapper.component.js file
import React, {Component} from "react";

import './currencies-wrapper.component.css'; // webpack (https://create-react-app.dev/docs/adding-a-stylesheet)

import Currencies from './currencies.component'; // As child...
import {Trainer} from './Globals';

export default class CurrenciesWrapper extends Component {
    static contextType = Trainer; // https://reactjs.org/docs/context.html#classcontexttype
    state = { /* ES7 instance attribute outside constructor -> supported by Babel */
        currency_of_interest_wrapper: null
    }

    get_currency_of_interest() {
        return this.state.currency_of_interest_wrapper;
    }

    read() { // Wrapper displays "$", "€", etc.
        return this.state.currency_of_interest_wrapper !== null ? this.state.currency_of_interest_wrapper.common_symbol : "";
    }

    render() { // Wrapper displays "$", "€", etc.
        return (
            <div className="currencies-wrapper">
                <h2>Currency of interest: {this.read()} ('{this.context}' as global var.)</h2>
                {/* Child: */} <Currencies currency_of_interest={this.get_currency_of_interest()}
                                           emit={this.update}/>
            </div>
        );
    }

    // Proxy for 'emit={this.update}':
    update = this._update.bind(this); /* ES7 instance attribute outside constructor -> supported by Babel */

    _update(currency) {
        // 'setState' enqueues changes to the component state and
        // tells React that this component and its children need to be re-rendered with the updated state...
        this.setState({currency_of_interest_wrapper: currency}); // Asynchronous!
    }
}
