// 'currencies-component.js' file
import React, {Component} from "react";

// https://create-react-app.dev/docs/adding-a-stylesheet/
import './currencies.component.css'; // webpack

import Currencies_service from './currencies-service';

class Description extends Component { // 'props' is injected by React through inheritance...
    /* ES7 instance attribute outside constructor -> supported by Babel: */
    change = ($event) => { // Lambda expression, 'this' cannot be something else that the "current" 'Description' instance:
        this.props.currency.description = $event.target.value;
    };
// https://www.robinwieruch.de/react-pass-props-to-component
    render() {
        if (this.props.currency.description)
            return <span>{this.props.currency.description}</span>;
        return <input onChange={this.change} placeholder="Enter some text" type="text"/>;
    }
}

class OtherCurrencies extends Component {
    render() {
        if (this.props.currency !== this.props.other_currency)
            return <span>
                <React.Fragment>
                    <input checked={this.props.checked(this.props.other_currency)}
                           id={this.props.other_currency.iso_code}
                           type="radio"/>
                    <label for={this.props.other_currency.iso_code}>{this.props.other_currency.common_symbol}</label>
                </React.Fragment>
            </span>;
        return (null); // Empty...
    }
}

export default class Currencies extends React.Component {
    static Default_substitute() {
        return Currencies.Currencies[Currencies.Euro];
    }

    // https://www.freecodecamp.org/news/this-is-why-we-need-to-bind-event-handlers-in-class-components-in-react-f7ea1a6f93eb/
    /* ES7 instance attribute outside constructor -> supported by Babel: */
    // send_ = (event) => { // Lambda expression, 'this' cannot be something else that the "current" 'Currencies' instance:
    //     window.alert(Object.getOwnPropertyNames(this).join(" - "));
    // };
    send = function (currency) { /* ES7 instance attribute outside constructor -> supported by Babel */
        // window.alert(JSON.stringify(this)); // 'this' is still bound to itself from 'this.send.bind(this, currency)'
        // Contextual currency must be emitted to wrapper as currency of interest...
        this.props.emit(currency); // https://ourcodeworld.com/articles/read/409/how-to-update-parent-state-from-child-component-in-react
        this.setState({currency_of_interest: currency}); // Record "I am" the currency of interest...
    }

    constructor(props, context) {
        super(props, context);
        // window.alert("this.props: " + Object.getOwnPropertyNames(this.props).join(" - ")); // 'currency_of_interest' - 'emit'
        // Web service is immediately launched:
        this._currencies_service = new Currencies_service(Currencies.Currencies[Currencies.Dollar], Currencies.Currencies[Currencies.Euro]);
        this.state = {
            currency_of_interest: this.props.currency_of_interest // From parent...
        }
    }

    checked(other_currency) {
        return this.state.currency_of_interest === other_currency;
    }

    color(currency) {
        return this.state.currency_of_interest === currency ? {color: "green"} : {color: "red"};
    }

    exchange_rate(currency) {
        if (!this.state.currency_of_interest) {
            window.alert('No currency of interest');
            return;
        }
        if (currency === this.state.currency_of_interest) {
            window.alert('From ' + currency.iso_symbol + ' To ' + this.state.currency_of_interest.iso_symbol + ' ' + 1.);
            return;
        }
        this._currencies_service.exchange_rate(currency, this.state.currency_of_interest).then(amount => {
            // this.setState({exchange_rate: amount}); // https://www.valentinog.com/blog/await-react/
            window.alert('From ' + currency.iso_symbol + ' To ' + this.state.currency_of_interest.iso_symbol + ' ' + amount);
        });
    }

    render() {
        return (
            <ul className="no-bullets">
                { // <- JSX evaluate exp.
                    Currencies.Currencies.map(currency => ( // <ul *ngFor="let currency of currencies">
                        // https://reactjs.org/docs/lists-and-keys.html#keys
                        <React.Fragment>
                            <li key={currency.iso_code}
                                onMouseOver={this.send.bind(this, currency)}
                                style={this.color(currency)} // Inline styling...
                                title={currency.common_symbol}>{currency.common_name}
                            </li>
                            (description:
                            <Description
                                currency={currency}/> {/* Data are passed in JSX syntax (HTML 'currency' custom attribute) -> 'props.currency' */}
                            )
                            <span class="buttons">
                                <button onClick={this.exchange_rate.bind(this, currency)} mat-fab>Exchange rate</button>
                            </span>
                            { // <- JSX evaluate exp.
                                Currencies.Currencies.map(other_currency => // <span *ngFor="let other_currency of currencies">
                                    <OtherCurrencies checked={this.checked.bind(this)} currency={currency}
                                                     other_currency={other_currency}/>
                                )
                            }
                        </React.Fragment>
                    ))
                }
            </ul>
        );
    }
}

Currencies.Currencies = [
    {
        common_symbol: "$",
        common_name: "Dollar",
        description: "First leading currency in the world...",
        iso_code: 840,
        iso_symbol: "USD",
        substitution_date: null
    },
    {
        common_symbol: "€",
        common_name: "Euro",
        description: "Second leading currency in the world...",
        iso_code: 978,
        iso_symbol: "EUR",
        substitution_date: null
    },
    {
        common_symbol: "Ls",
        common_name: "Lats",
        description: undefined,
        iso_code: 428,
        iso_symbol: "LVL",
        substitute: {
            common_symbol: "€",
            common_name: "Euro",
            description: "Second leading currency in the world...",
            iso_code: 978,
            iso_symbol: "EUR",
            substitution_date: null
        },
        substitution_date: Date.parse('01 Jan 2014 00:00:00 GMT')
    }
];
Currencies.Dollar = 0;
Currencies.Euro = 1;


